function get_angles_final,azi,yy,mm,dd,hh,earth = earth

;+
;==============================================================================
;
;	function: get_angles_final
;
;	purpose : return the incidence and rotation angles for the
;	          choosen azimuth of first mirror and date
;                 Additionally polarizer rotation for tel.cal.data
;
;       written : cbeck@KIS, 11/01
;
;==============================================================================
;
;       Check number of parameters.
;
;==============================================================================
if n_params() lt 5 then begin
	print
	print, "usage:	angles = get_angles_final(azi,yy,mm,dd,hh)"
	print
        print, "        Calculate all needed angles from coelostat settings."
        print, "        Return i1,th1,i2,th2,i3,th3,i4 as in get_angles.pro."
        print, "        Also returned : th_pol_C1,th_C1_pol,th_pol_C2."
        print, "        Needed only for polarizer on C1."
        print, "        Calculation is partly analytically/numerically."
	print
	print, "	Arguments"
        print, "                azi : azimuth of first mirror, positive values"
        print, "                      towards east"
        print, "                yy,mm,dd,hh : date and time in the form"
        print, "                      01,06,12,12.345 UT"
	print
        print, "        Keywords"
        print, "                /earth : RF0 fixed on earth"
        print
        print, "                Output angles are in degrees."
        print, "        Output"
        print, "                vector(10),[i1,th1,i2,th2,i3,th3,i4,$"
        print, "                th_pol_C1,th_C1_pol,th_pol_C2]"
        print, "                i1,i2,i3  : incidence angles on C1, C2, main"
	print, "                th1       : rotation between RF0 and C1"
        print, "                th2       : rotation between C1 nad C2"
        print, "                th3       : rotation between C2 and main"
	print, "                th_pol_C1 : rotation between polarizer and C1"
	print, "                th_C1_pol : rotation between reflected beam of"
        print, "                            C1 and polarizer"
	print, "                th_pol_C2 : rotation between polarizer and C2"
        print
	return, 0
endif
;==============================================================================
;-

;------------------------------------------------------------------
; geographical latitude of Tenerife
;------------------------------------------------------------------
phi = (28.+18./60.+8.9/3600.)

;------------------------------------------------------------------
; conversion factors to/from rad to hour/degrees
;------------------------------------------------------------------
radtohour = 12./!pi
radtodeg = 180./!pi
degtorad = 1./radtodeg
hourtorad = 1./radtohour

;------------------------------------------------------------------
; convert input angles
;------------------------------------------------------------------
phirad = phi * degtorad
azrad  = azi * degtorad

;---------------------------------------------------------------------------
; get sun in equatorial coordinates declination, hour angle, use
; r_frame_asp_mod (/lapalma = Tenerife)
;---------------------------------------------------------------------------
r_frame_asp_mod,0,0,yy,mm,dd,hh,0,0,d1,d2,d3,d4,d5,d6,d7,d8,raSun,decSun,d9,b0,p,d10,d11,par,haSun,/lapalma

declsun = decSun    ; declination of the sun
hoursun = haSun     ; hour angle of the sun

;---------------------------------obsolete-------------------------------------
; control height by using coelostatmod (optional)
;---------------------------------------------------------------------------
;zz = coelostatmod(haSun*12/!pi,decSun*180/!pi,-azi)
;rr=zz(2)+77
;---------------------------------obsolete-------------------------------------

;---------------------------------------------------------------------------
; 1. Get everything needed about second coelostat mirror C2
;---------------------------------------------------------------------------

;---------------------------------------------------------------------------
; coordinates of second mirror are azimuth az_C2 (horizontal plane
; system), angular height  -declsun (equatorial plane system), both
; systems centered on middle of C1, az_C2 positive from S to W
; find position in horizontal plane coordinates angular height h_C2, az_C2
;---------------------------------------------------------------------------
h = findgen(10000)*!pi/2./10000.
x = sin(phirad)*sin(h)-cos(phirad)*cos(azrad)*cos(h)

;---------------------------------------------------------------------------
; control of result (optional)
; activate to control for existence of intersection point
;---------------------------------------------------------------------------
;plot,h,x,ystyle=1,yrange=[-2,2]
;

;---------------------------------------------------------------------------
; find intersection point
;---------------------------------------------------------------------------
zz = where(x gt sin(-decSun)-.001 and x le sin(-decSun)+.001)

if zz(0) eq -1 then begin
if !d.window eq -1 then window,0,title='!! Warning !!',xsize=300,ysize =100,xpos = 20,ypos = 150
erase
xyouts,0,.8,'No sun observation possible with this settings !',z=1.4,font=0,/normal
xyouts,0,.6,'Please check used azimut or date !',z=1.4,font=0,/normal
xyouts,0,.4,'Warning : pillar height',z=1.4,font=0,/normal
print,'Warning : invalid pillar height, time '+num2string(hh)
endif

h_C2 = mean(zz)*!pi/2./10000.
az_C2 = azrad

;---------------------------------------------------------------------------
; calculate height over C1 for comparison with rr from above (optional)
;---------------------------------------------------------------------------
hhh = 155*tan(h_C2)
if hhh-88 gt 250 then begin
print,' Warning : invalid pillar height, time '+num2string(hh)
print,' Pillar at : ',hhh-88,' cm'
print,' Please check azimuth and date.'
endif


;---------------------------------------------------------------------------
; second mirror in horizontal CARTESIAN coordinates : x_C2, y_C2, z_C2
; the x-axis is positive towards S, y-axis positive towards E and z
; positive to zenith
; !!!! ATTENTION : now the angle in the x-y-plane is measured positive
; from S to E !!! Therefore azrad -> -azrad
;---------------------------------------------------------------------------
y_C2 = sin(-azrad)*sin(!pi/2.-h_C2)
x_C2 = cos(azrad)*sin(!pi/2.-h_C2)
z_C2 = cos(!pi/2.-h_C2)

;---------------------------------------------------------------------------
; calculation of mirror normal on C2 : x_N2, y_N2, z_N2
; mirror normal is in the middle between -[x_C2,y_C2,z_C2] and [0,0,-1],
; which are both normalized unit vectors. Therefore one can get the mirror
; normal simply by adding both and the normalizing result (isosceles 
; triangle)  
;---------------------------------------------------------------------------
x_N2 = -x_C2
y_N2 = -y_C2
z_N2 = -z_C2-1

;---------------------------------------------------------------------------
; normalize it
;---------------------------------------------------------------------------
hivec = [x_N2,y_N2,z_N2]
hivec= hivec/float(vecnorm(hivec))

x_N2 = hivec(0)
y_N2 = hivec(1)
z_N2 = hivec(2)
;---------------------------------------------------------------------------
; Done C2.
;---------------------------------------------------------------------------

;---------------------------------------------------------------------------
; 2. calculate Sun in horizontal coordinates h_sun, az_sun and
; cartesian x_sun, y_sun, z_sun, use transformation equations from d,t
; -> h,A (equi -> hor)
;---------------------------------------------------------------------------
sin_height_hor_sun = cos(decSun)*cos(haSun)*cos(phirad) + sin(decSun)*sin(phirad)
height_hor_sun = asin(sin_height_hor_sun)
cos_height_hor_sun = cos(height_hor_sun)

sin_az_sun = (sin(haSun)*cos(decSun)) / cos_height_hor_sun
cos_az_sun = ( cos(decSun)*cos(haSun)*sin(phirad) - sin(decSun)*cos(phirad) )/ cos_height_hor_sun

az_sun = atan(sin_az_sun,cos_az_sun)
;---------------------------------------------------------------------------
; use horizontal plane coordinates A,h to get cartesian values as
; above, h is horizontal height -> !pi/2. - h for spherical coordinates
;---------------------------------------------------------------------------
hhhh = height_hor_sun*radtodeg

y_sun = sin(-az_sun)*sin(!pi/2.-height_hor_sun)
x_sun = cos(az_sun)*sin(!pi/2.-height_hor_sun)
z_sun = cos(!pi/2.-height_hor_sun)
;---------------------------------------------------------------------------
; Done Sun.
;---------------------------------------------------------------------------

;---------------------------------------------------------------------------
; 3. Coelostat mirror C1
;---------------------------------------------------------------------------
; calculate postion of mirror normal 1 : add sun pos. and C2 pos, normalize,
; result is in cartesian coordinates x_N1, y_N1, z_N1
;---------------------------------------------------------------------------
x_N1a = x_sun+x_C2
y_N1a = y_sun+y_C2
z_N1a = z_sun+z_C2

hivec = [x_N1a,y_N1a,z_N1a]
hivec= hivec/float(vecnorm(hivec))

x_N1 = hivec(0)
y_N1 = hivec(1)
z_N1 = hivec(2)

;---------------------------------------------------------------------------
; get horizontal plane coordinates h_N1, az_N1 (optional for comparison)
;---------------------------------------------------------------------------
;aa = (hivec(0)^2.+hivec(1)^2.)/hivec(2)^2.
;h_N1 = !pi/2.-atan(sqrt(aa))
;az_N1 = -atan(hivec(1)/hivec(0))
;---------------------------------------------------------------------------
; Done C1.
;---------------------------------------------------------------------------

;---------------------------------------------------------------------------
; 4. incidence angle on first mirror i1 : scalar product of N1 and C2 (equal
; to scalar product of N1 and sun, print both values for comparison
;---------------------------------------------------------------------------
i1a = acos(transpose([x_C2,y_C2,z_C2])#[x_N1,y_N1,z_N1])
;i1b = acos(transpose([x_N1,y_N1,z_N1])#[x_sun,y_sun,z_sun])
i1 = i1a*radtodeg

; control result by printing values 
;print,i1b*radtodeg,i1a*radtodeg,' i1'

;---------------------------------------------------------------------------
; 5. incidence angle on second mirror i2 : scalar product of N2 and C2 (equal
; to scalar product of N2 and (0,0,-1) equal to (!pi/2.-h_C2)/2.)
;---------------------------------------------------------------------------
i2 = (!pi/2.-h_C2)/2.*radtodeg
;i2a = acos(transpose([x_N2,y_N2,z_N2])#(-[x_C2,y_C2,z_C2]))
;i2b = acos(transpose([0,0,-1])#[x_N2,y_N2,z_N2])

; control result by printing values 
;print,i2,i2a*radtodeg,i2b*radtodeg,' i2'
;---------------------------------------------------------------------------

;---------------------------------------------------------------------------
;                  Now construct the reference frames 
;---------------------------------------------------------------------------

;---------------------------------------------------------------------------
; 6. coelostat mirror 1 : x1 ,y1, z1 (axis VECTORS (3) )
;---------------------------------------------------------------------------
z1a = [x_sun,y_sun,z_sun] ; direction OPPOSITE to propagation
n1a = [x_N1,y_N1,z_N1]    ; normal on mirror 1

;---------------------------------------------------------------------------
; construction of the x-axis : n,z,x are in the SAME plane. Therefore
; one can decompose n using n = <n,z> * z + beta, <beta,z> = 0 -->
; beta = n - <n,z> * z, normalize beta -> beta/norm = x
;---------------------------------------------------------------------------
a = transpose(n1a)#z1a ; scalar product of z and n
beta = n1a-a(0)*z1a 
x1a = beta/float(vecnorm(beta))
z1a = -z1a            ; now change z into propagation direction

;---------------------------------------------------------------------------
; y-axis = (x-axis) x (z-axis) to get right-handed-reference frame
;---------------------------------------------------------------------------
y1a = crossprod(x1a,z1a)

;---------------------------------------------------------------------------
; 7. coelostat mirror 2 : x2 ,y2, z2, same procedure
;---------------------------------------------------------------------------
z2a = -[x_C2,y_C2,z_C2] ; direction OPPOSITE to propagation
n2a = [x_N2,y_N2,z_N2]    ; normal on mirror 2

;---------------------------------------------------------------------------
; construction of the x-axis : n = <n,z> * z + beta, <beta,z> = 0 -->
; beta = n - <n,z> * z, normalize
;---------------------------------------------------------------------------
a = transpose(n2a)#z2a ; scalar product of z and n
beta = n2a-a(0)*z2a 
x2a = beta/float(vecnorm(beta))
z2a = -z2a            ; change z into propagation direction
; y-axis = (x-axis) x (z-axis)
y2a = crossprod(x2a,z2a)

;---------------------------------------------------------------------------
; Done RF1, RF2. RF3 is fixed, RF_first is fixed/constructed below.
;---------------------------------------------------------------------------

;---------------------------------------------------------------------------
; construct RF of linear polarizer on C1
;---------------------------------------------------------------------------
x_pol = [-cos(phirad),0,sin(phirad)]
z_pol = -n1a
y_pol = crossprod(x_pol,z_pol)

;---------------------------------------------------------------------------
; 8. Now rotate the reference frames both to nearest z-axis
;---------------------------------------------------------------------------

;---------------------------------------------------------------------------
; reference frame 1 : change z-axis into spherical coordinates
;---------------------------------------------------------------------------
tanphi1 = z1a(1)/z1a(0)
tan2th1 = (z1a(0)^2.+z1a(1)^2.)/z1a(2)^2.
ee = sqrt(tan2th1)
phi1 = atan(tanphi1)
th1 = atan(ee)  

; equal to th1 = !pi/2. - height_hor_sun
; not to be confused with rotation angle th1, this is spherical
; coordinates theta angle to cartesian z-axis [0,0,1]

;---------------------------------------------------------------------------
; correct results of atan for the different quadrants (atan ->
; [-pi,pi]), but correct azimuthal angle phi is wanted
;---------------------------------------------------------------------------
; III
if (z1a(0) lt 0) and (z1a(1) lt 0) then phi1 = phi1+!pi
; II
if (z1a(0) lt 0) and (z1a(1) gt 0) then phi1 = phi1+!pi
; IIII
if (z1a(0) gt 0) and (z1a(1) lt 0) then phi1 = phi1+2*!pi

;---------------------------------------------------------------------------
; reference frame 2 : change z-axis into spherical coordinates (easier)
;---------------------------------------------------------------------------
phi2 = 2*!pi-azrad 
th2 = !pi/2.-h_C2

;---------------------------------------------------------------------------
; reference frame of polarizer : change z-axis into spherical coordinates
;---------------------------------------------------------------------------
tanphi_pol = z_pol(1)/z_pol(0)
tan2th_pol = (z_pol(0)^2.+z_pol(1)^2.)/z_pol(2)^2.
ee = sqrt(tan2th_pol)
phi_pol = atan(tanphi_pol)
th_pol = atan(ee)  

;---------------------------------------------------------------------------
; correct results of atan for the different quadrants (atan ->
; [-pi,pi]), but correct azimuthal angle phi is wanted
;---------------------------------------------------------------------------
; III
if (z_pol(0) lt 0) and (z_pol(1) lt 0) then phi_pol = phi_pol+!pi
; II
if (z_pol(0) lt 0) and (z_pol(1) gt 0) then phi_pol = phi_pol+!pi
; IIII
if (z_pol(0) gt 0) and (z_pol(1) lt 0) then phi_pol = phi_pol+2*!pi


;---------------------------------------------------------------------------
; Now rotate z1a onto [0,0,-1]
;---------------------------------------------------------------------------
; rotation angle to bring x-axis onto normal to connection of z1 and [0,0,-1]
;---------------------------------------------------------------------------
ff = phi1-!pi/2. ; as normal to connection = (-sin(phi1),cos(phi1),0)
sss = sin(ff)
ccc = cos(ff)

dz1 = [[ccc,-sss,0],[sss,ccc,0],[0,0,1]]         ; rotation
dz1_invers = [[ccc,sss,0],[-sss,ccc,0],[0,0,1]]  ; inverse of rotation

;---------------------------------------------------------------------------
; second rotation angle : rotation around axis from step above for
; angle -th
;---------------------------------------------------------------------------
dreh1 = -th1

ss = sin(dreh1)
cc = cos(dreh1)
dx = [[1,0,0],[0,cc,-ss],[0,ss,cc]]

;---------------------------------------------------------------------------
; set the complete rotation : z1a -> [0,0,-1]
;---------------------------------------------------------------------------
D = dz1 ## dx ## dz1_invers 

y1a_old = y1a

;---------------------------------------------------------------------------
; rotate x1-, y1- and z1-axis
;---------------------------------------------------------------------------
x1a = D##x1a
y1a = D##y1a
z1a = D##z1a

;---------------------------------------------------------------------------
; Now rotate z_pol onto [0,0,-1]
;---------------------------------------------------------------------------
; rotation angle to bring x-axis onto normal to connection of z_pol
; and [0,0,-1] 
;---------------------------------------------------------------------------
ff = phi_pol-!pi/2. ; as normal to connection = (-sin(phi1),cos(phi1),0)
sss = sin(ff)
ccc = cos(ff)

dz1 = [[ccc,-sss,0],[sss,ccc,0],[0,0,1]]         ; rotation
dz1_invers = [[ccc,sss,0],[-sss,ccc,0],[0,0,1]]  ; inverse of rotation

;---------------------------------------------------------------------------
; second rotation angle : rotation around axis from step above for
; angle -th
;---------------------------------------------------------------------------
dreh1 = -th_pol

ss = sin(dreh1)
cc = cos(dreh1)
dx = [[1,0,0],[0,cc,-ss],[0,ss,cc]]

;---------------------------------------------------------------------------
; set the complete rotation : z1a -> [0,0,-1]
;---------------------------------------------------------------------------
D = dz1 ## dx ## dz1_invers 

x_pol_old = x_pol
;---------------------------------------------------------------------------
; rotate x1-, y1- and z1-axis
;---------------------------------------------------------------------------
x_pol = D##x_pol
y_pol = D##y_pol
z_pol = D##z_pol

;---------------------------------------------------------------------------
; rotate z2a onto [0,0,1], same as above
;---------------------------------------------------------------------------
; rotation angle to bring x-axis onto normal to connection of z2a and [0,0,1]
;---------------------------------------------------------------------------
ff = phi2-!pi/2. ; as normal to connection = (-sin(phi2),cos(phi2),0)
sss = sin(ff)
ccc = cos(ff)

dz1 = [[ccc,-sss,0],[sss,ccc,0],[0,0,1]]         ; rotation
dz1_invers = [[ccc,sss,0],[-sss,ccc,0],[0,0,1]]  ; inverse of rotation

;---------------------------------------------------------------------------
; second rotation angle
;---------------------------------------------------------------------------
dreh1 = th2

ss = sin(dreh1)
cc = cos(dreh1)
dx = [[1,0,0],[0,cc,-ss],[0,ss,cc]]

;---------------------------------------------------------------------------
; complete rotation : z2a -> [0,0,1]
;---------------------------------------------------------------------------
D = dz1 ## dx ## dz1_invers 

;---------------------------------------------------------------------------
; rotate x2a-, y2a- and z2a-axis
;---------------------------------------------------------------------------
x2a = D##x2a
y2a = D##y2a
z2a = D##z2a

;---------------------------------------------------------------------------
; get azimuthal angle of rotated x1a, x2a for angle between them 
;
; reference frame 1 : get azimuthal angle of x1a
;---------------------------------------------------------------------------
phix1 = atan(x1a(1)/x1a(0))

;---------------------------------------------------------------------------
; correct results of atan for the different quadrants (atan -> [-pi,pi])
;---------------------------------------------------------------------------
; III
if (x1a(0) lt 0) and (x1a(1) lt 0) then phix1 = phix1+!pi
; II
if (x1a(0) lt 0) and (x1a(1) gt 0) then phix1 = phix1+!pi
; IIII
if (x1a(0) gt 0) and (x1a(1) lt 0) then phix1 = phix1+2*!pi

;---------------------------------------------------------------------------
; reference frame polarizer : get azimuthal angle of x_pol
;---------------------------------------------------------------------------
;phix_pol = atan(x_pol(1)/x_pol(0))

;---------------------------------------------------------------------------
; correct results of atan for the different quadrants (atan -> [-pi,pi])
;---------------------------------------------------------------------------
; III
;if (x_pol(0) lt 0) and (x_pol(1) lt 0) then phix_pol = phix_pol+!pi
; II
;if (x_pol(0) lt 0) and (x_pol(1) gt 0) then phix_pol = phix_pol+!pi
;; IIII
if (x_pol(0) gt 0) and (x_pol(1) lt 0) then phix_pol = phix_pol+2*!pi

;---------------------------------------------------------------------------

;---------------------------------------------------------------------------
; construct RF1_out, rotate to [0,0,1]
;---------------------------------------------------------------------------
z1b = [x_C2,y_C2,z_C2]

a = transpose(n1a)#z1b ; scalar product of z and n
beta = n1a-a(0)*z1b 
x1b = beta/float(vecnorm(beta))

;---------------------------------------------------------------------------
; y-axis = (x-axis) x (z-axis) to get right-handed-reference frame
;---------------------------------------------------------------------------
y1b = crossprod(x1b,z1b)

ff = phi2-!pi/2. ; as normal to connection = (-sin(phi2),cos(phi2),0)
sss = sin(ff)
ccc = cos(ff)

dz1 = [[ccc,-sss,0],[sss,ccc,0],[0,0,1]]         ; rotation
dz1_invers = [[ccc,sss,0],[-sss,ccc,0],[0,0,1]]  ; inverse of rotation

;---------------------------------------------------------------------------
; second rotation angle
;---------------------------------------------------------------------------
dreh1 = th2

ss = sin(dreh1)
cc = cos(dreh1)
dx = [[1,0,0],[0,cc,-ss],[0,ss,cc]]

;---------------------------------------------------------------------------
; complete rotation : z1b -> [0,0,1]
;---------------------------------------------------------------------------
D = dz1 ## dx ## dz1_invers 

y1b_old = y1b

z1b = D ## z1b
x1b = D ## x1b
y1b = D ## y1b

phix1b = atan(x1b(1)/x1b(0))
;---------------------------------------------------------------------------
; Done RF1_out.
;---------------------------------------------------------------------------

;---------------------------------------------------------------------------
; correct results of atan for the different quadrants (atan -> [-pi,pi])
;---------------------------------------------------------------------------
; III
if (x1b(0) lt 0) and (x1b(1) lt 0) then phix1b = phix1b+!pi
; II
if (x1b(0) lt 0) and (x1b(1) gt 0) then phix1b = phix1b+!pi
; IIII
if (x1b(0) gt 0) and (x1b(1) lt 0) then phix1b = phix1b+2*!pi

;---------------------------------------------------------------------------

;---------------------------------------------------------------------------
; reference frame 2 : get azimuthal angle of x2a
;---------------------------------------------------------------------------
phix2 = atan(x2a(1)/x2a(0))

;---------------------------------------------------------------------------
; correct results of atan for the different quadrants (atan -> [-pi,pi])
;---------------------------------------------------------------------------
; III
if (x2a(0) lt 0) and (x2a(1) lt 0) then phix2 = phix2+!pi
; II
if (x2a(0) lt 0) and (x2a(1) gt 0) then phix2 = phix2+!pi
; IIII
if (x2a(0) gt 0) and (x2a(1) lt 0) then phix2 = phix2+2*!pi

;print,phix1*radtodeg,'phix1'
;print,phix2*radtodeg,'phix2'

;---------------------------------------------------------------------------
; rotation around [0,0,1]=z1a=z2a to bring x1b onto x2a
;---------------------------------------------------------------------------

dreh3= (phix2-phix1b)

;---------------------------------------------------------------------------
; 9. rotation angle to bring RF1 and RF2 into coincidence
;---------------------------------------------------------------------------
theta2 = dreh3*radtodeg

;---------------------------------------------------------------------------
; control of result by actually doing the rotation, give out warning
; if x-axes after rotation differ by more than 2*10^-5 (can happen
; if a) sun is below horizon b) 2*10^-6 almost always arise)
;---------------------------------------------------------------------------
ff = dreh3
sss = sin(ff)
ccc = cos(ff)
dz = [[ccc,-sss,0],[sss,ccc,0],[0,0,1]]         ; rotation

;---------------------------------------------------------------------------
; give warning
;---------------------------------------------------------------------------
if total(abs(transpose(dz##x1b-x2a))) gt 2*10^(-5.) then begin
print,transpose(dz##x1b-x2a),' Difference of x-axes C1->C2'
print,'Warning ! C1->C2 at '+num2string(hh)+'h'
print,'Sun heigth above horizon : '+num2string(height_hor_sun)
endif

;---------------------------------------------------------------------------
; print values to check for correctness : z1a,z2a pointing up, x2a,
; y2a fixed, x1a(1) = y-component must be zero, when sun is standing
; directly BEHIND C2 (midday for azrad = 0) (optional)
;---------------------------------------------------------------------------
;print,transpose(z2a),' z2a'
;print,transpose(z1a),' z1a'
;print,transpose(x2a)
;print,transpose(x1a)
;print,transpose(y2a)
;print,transpose(y1a)
;print,phix1*radtodeg,' phix1'
;print,phix2*radtodeg,' phix2'
;print,dreh3*radtodeg,' theta2'
;print,th1*radtodeg,' th1'
;print,th2*radtodeg,' th2'

;---------------------------------------------------------------------------
; 3. coordinate system : main mirror (fixed), only for completeness
;---------------------------------------------------------------------------
z3 = [0,0,-1]
x3 = [-1,0,0]
y3 = [0,-1,0]
phix3 = !pi

;---------------------------------------------------------------------------
; 10. rotation between RF2 (fixed by azrad) and RF3 (fixed)
;---------------------------------------------------------------------------
theta3 = (phix3-phix2)*radtodeg

;---------------------------------------------------------------------------
; control of result by performing rotation
;---------------------------------------------------------------------------
ff = theta3*degtorad

sss = sin(ff)
ccc = cos(ff)
dz = [[ccc,-sss,0],[sss,ccc,0],[0,0,1]]         ; rotation

if total(abs(transpose(dz##x2a-[-1,0,0]))) gt 2*10^(-5.) then begin
print,'Warning ! C2->main'
print,transpose(dz##x2a-[-1,0,0]),' Difference of x-axes C2->main'
endif

;---------------------------------------------------------------------------
; construct RF0
;---------------------------------------------------------------------------

;---------------------------------------------------------------------------
; first reference frame : +Q = north =[-1,0,0],  +y = west =[0,-1,0], +z
; pointing up used for /earth
; else use tangent to equatorial plane at the position of the sun hour
; angle 
;---------------------------------------------------------------------------
yfirst=[0,-1,0] ; for /earth, not used
zfirst=[0,0,1]  ; for /earth, not used

;---------------------------------------------------------------------------
; definition of solar xfirst
;---------------------------------------------------------------------------
xfirst = [-sin(phirad)*sin(haSun),-cos(haSun),-cos(phirad)*sin(haSun)]

;---------------------------------------------------------------------------
; rotate the same way as z1a for comparison
;---------------------------------------------------------------------------
ff = phi1-!pi/2. ; as normal to connection = (-sin(phi1),cos(phi1),0)
sss = sin(ff)
ccc = cos(ff)
dz1 = [[ccc,-sss,0],[sss,ccc,0],[0,0,1]]         ; rotation
dz1_invers = [[ccc,sss,0],[-sss,ccc,0],[0,0,1]]  ; inverse of rotation
dreh1 = -th1
ss = sin(dreh1)
cc = cos(dreh1)
dx = [[1,0,0],[0,cc,-ss],[0,ss,cc]]
D = dz1 ## dx ## dz1_invers 

xfirst = D ## xfirst

;---------------------------------obsolete------------------------------------
; definition of xfirst as tangent to the horizontal plane 
;-----------------------------------------------------------------------------
; xfirst = [sin(az_sun),-cos(az_sun),0]
;---------------------------------obsolete-------------------------------------

phifirst = atan(xfirst(1)/xfirst(0))
; III
if (xfirst(0) lt 0) and (xfirst(1) lt 0) then phifirst = phifirst+!pi
; II
if (xfirst(0) lt 0) and (xfirst(1) gt 0) then phifirst = phifirst+!pi
; IIII
if (xfirst(0) gt 0) and (xfirst(1) lt 0) then phifirst = phifirst+2*!pi

;---------------------------------------------------------------------------
; if /earth use fixed RF0 on earth
;---------------------------------------------------------------------------
;if n_elements(earth) ne 0 then begin
;xfirst=[-1,0,0]
;phifirst = !pi
;endif

;---------------------------------------------------------------------------
; 11. rotation between RF_first and RF1
;---------------------------------------------------------------------------
rotfirst = phix1 - phifirst
thetafirst = -rotfirst*radtodeg

;---------------------------------------------------------------------------
; control of result
;---------------------------------------------------------------------------
ff =  rotfirst  
sss = sin(ff)
ccc = cos(ff)
dz = [[ccc,-sss,0],[sss,ccc,0],[0,0,1]]         ; rotation

if total(abs(transpose(dz##xfirst-x1a))) gt 2*10^(-5.) then begin
print,transpose(dz##xfirst-x1a),' Difference of x-axes first->C1'
print,'Warning ! first->C1 at '+num2string(hh)+'h'
print,'Sun heigth above horizon : '+num2string(height_hor_sun)
endif

;---------------------------------------------------------------------------
; control rotation of z1a, z2a,z_pol
;---------------------------------------------------------------------------

if total(abs(transpose(z1a-[0,0,-1]))) gt 2*10^(-5.) then begin
print,transpose(z1a-[0,0,-1]),' Difference of z-axes z1a->0,0,-1'
print,'Warning ! z1a'
endif
if total(abs(transpose(z_pol-[0,0,-1]))) gt 2*10^(-5.) then begin
print,transpose(z_pol-[0,0,-1]),' Difference of z-axes z_pol->0,0,-1'
print,'Warning ! z_pol'
endif
if total(abs(transpose(z2a-[0,0,1]))) gt 2*10^(-5.) then begin
print,transpose(z2a-[0,0,1]),' Difference of z-axes z2a->0,0,1'
print,'Warning ! z2a'
endif

;---------------------------------------------------------------------------
; other needed values (fixed)
;---------------------------------------------------------------------------
i3 = 0.84
i4 = i3

th1 = thetafirst+90.
th2 = theta2
th3 = 90-azi

;---------------------------------------------------------------------------
while (th1 lt 0) do   th1=th1+360 
while (th1 gt 360) do th1=th1-360 
while (th2 lt 0) do   th2=th2+360 
while (th2 gt 360) do th2=th2-360 
while (th3 lt 0) do   th3=th3+360 
while (th3 gt 360) do th3=th3-360
;---------------------------------------------------------------------------

;---------------------------------------------------------------------------
; return all angles
;---------------------------------------------------------------------------

return,[i1,th1,i2,th2,i3,th3,i4,th1,th1,th2-th1]

end
