pro iterate_gain,shift_val,i_line,mean_line,gain_line,xswitch,wave_lim,plt = plt,err_index = err_index,verbose = verbose

;+
;==============================================================================
;
;	procedure : iterate_gain
;
;	purpose   : find optimal shift vale by least-square-fit to minimize
;	            deviation of gaintable line from straight line
;
;       calls     : analytger.pro
;
;       written   : cbeck@KIS 3/03 
;
;==============================================================================
;
;	Check number of parameters.
;
;==============================================================================
if n_params() lt 6 then begin
	print
	print, "usage:  iterate_gain,shift_val,i_line,mean_line,gain_line,"
        print, "                     xswitch,wave_lim,plt = plt,"
        print, "                     err_index = err_index"
        print
	print, "	Find the optimal shift value for mean profile by"
        print, "        minimizing deviation of gaintable line from a"
        print, "        straight line."
      	print
	print, "	Arguments"
   	print, "        Input:"
	print, "	     i_line     : line of flat field image"
        print, "             mean_line  : mean profile"
        print, "             gain_line  : rough gainline by division"
        print, "             xswitch    : switch to modify speed"
        print, "             wave_lim   : wave_lim = [low,high] fixes spectral"
        print, "                          range for least-square fit to shift."
        print, "                          wave_lim = [low,high,speed], speed"
        print, "                          controls convergence of fit,"
        print, "                       0.5 < speed < 4."
        print, "        Output:"
        print, "               shift_val : iterated shift value"
	print
	print, "	Keywords"
        print, "	       verbose	 : give run-time info"
	print
	return
endif
;==============================================================================
;-

if xswitch eq 42 then scale = 1/1000000. else scale = 1

; store old shift value
oldval = shift_val
n = length(mean_line)

pp = 1
if pp eq 1 then begin

speed = wave_lim(2)
wave_start = wave_lim(0)
wave_end = wave_lim(1)


xx = findgen(wave_end-wave_start+1)
helpline  = fltarr(n,2)
helpline(*,0) = mean_line

; do plotting
if n_elements(plt) ne 0 then begin
   window,0
   plot,i_line,/ynozero
   oplot,mean_line
   oplot,[wave_start,wave_start],[0,50000]
   oplot,[wave_end,wave_end],[0,50000]
   window,1
endif

erg = analytger(xx+wave_start,gain_line(wave_start:wave_end))
yy = erg(1)+erg(0)*findgen(n)

if n_elements(plt) ne 0 then begin
   wset,1
   plot,gain_line(wave_start:wave_end),/ynozero
   oplot,yy(wave_start:wave_end)
   paused
endif

qual = 1
chiold = 0
h = .001

;if firsty eq 42 then h = .0001

; minimization loop
index  = 0

; principal break condition: no change of chisquare down to numerical precision
while (qual ne chiold) do begin

; break condition: chi square increases
if index gt 2 and chiold gt qual then break

; break condition: shift value larger than 3 pixels
if abs(shift_val-oldval) gt 3 then break

qual = chiold

; chi square of previous step
chiold = total((gain_line(wave_start:wave_end)-yy(wave_start:wave_end))^2.)

shift_valold = shift_val

; increase shift
shift_val = shift_val + h
shift_line = shift_frac(helpline,shift_val,0,'C')
gain_line = i_line/shift_line(*,0)

; new chi square
chi = total((gain_line(wave_start:wave_end)-yy(wave_start:wave_end))^2.)

deltachi = chi-chiold
gradient = deltachi/h
stepwidth = abs(deltachi/chiold)
shift_val = shift_valold

shift_val = shift_val -gradient*stepwidth*100.*scale
shift_line = shift_frac(helpline,shift_val,0,'C')
gain_line = i_line/float(shift_line(*,0))

erg = analytger(xx+wave_start,gain_line(wave_start:wave_end))
yy = erg(1)+erg(0)*findgen(n)

;print,index,gradient*stepwidth*100.*scale,deltachi

index = index+1

; break condition: more than 4000 steps
;if index gt 4000 then break
if index lt 2 and xswitch eq 42 then scale = speed

if n_elements(plt) ne 0 and  index mod 100 eq 0 then begin  ; and  index mod 100 eq 0
wset,1
erase
plot,gain_line(wave_start:wave_end),/ynozero
oplot,yy(wave_start:wave_end)
paused
endif

endwhile

if n_elements(verbose) ne 0 then begin
   print,'Iteration steps: ',index
   if index lt 4 or index gt 4000 then begin
      print,' iterate_gain: Warning ! Loop broken after '+num2string(index)+' steps !'
      print,' Please check the speed variable: wave_lim =[x1,x2,speed] !'

      if n_elements(err_index) ne 0 then err_index = err_index+1
endif

endif

if pp eq 0 then begin
   helpline1 = fltarr(n,10)
   helpline2 = helpline1
   for k = 0,9 do begin
      helpline1(*,k) = mean_line
      helpline2(*,k) = i_line
   endfor
   
   erg1 = shc(helpline2,helpline1,/interp,filter=2)
   shift_val =erg1(0) 
   
;if shift_val gt 4 then  shift_val = oldval

;if abs(shift_val-oldval) gt 2. then  shift_val = oldval

   if abs(shift_val-oldval) gt .5 then  shift_val = oldval

endif

if n_elements(verbose) ne 0 then print,'Final shift correction: ', shift_val-oldval
endif

; use old value if new one differs by more than 0.5 pixels
if abs(shift_val-oldval) gt .5 then  shift_val = oldval

end
